searchNodes=[{"doc":"Contains the basic functions needed to communicate with CosmosDb. Configuration There are specific configuration values that must be present in order to communicate with your instance of Cosmos Db. They are: COSMOS DB KEY - This is either the primary or secondary Read-Write key of your database. You can find these keys under the 'Keys' tab of your databases settings. This value can be set in your apps config using :cosmos_db_key or as an environment variable named COSMOS_DB_KEY . COSMOS DB HOST URL - This is the URI to your Cosmos DB instance. The value can be found under the Essentials section of your Cosmos Db Overview. This value can be set in your apps config using :cosmos_db_host_url or as an environment variable named COSMOS_DB_HOST_URL . Example using your apps configuration settings: config :cosmos_db_ex , cosmos_db_key : &quot;{your_primary_or_secondary_key_here}&quot; , cosmos_db_host_url : &quot;https://your-cosmos-db.documents.azure.com/&quot; , Remember that your keys are secrets that should never be saved in any version control system. If you're going to use the config option please take caution and look at using a config provider that retrieves the secrets from a vault or the environment.","ref":"CosmosDbEx.html","title":"CosmosDbEx","type":"module"},{"doc":"Creates a new document in the specified database and container. Documents can be any struct that implements the CosmosDbEx.Client.Documents.Identifiable protocol, or any struct or map that contains an id field. The Identifiable protocol contains a single function that must be implemented, called get_id(). You can return a string in any format to represent the id given to CosmosDb. NOTE: You must implement the Jason.Encoder protocol for any struct that will be used as the document being created. You can do this by adding the following to your struct definition: @dervie { Jason.Encoder , only : [ ... . ] } defstruct ... Every request will return a tuple containing the status of the request as well as any information Cosmos Db returned in the body of the response. The only exception is when our call to the Rest API fails for a non-CosmosDb related issue Here are the tuple pairs returned for the following situations: {:ok, %CosmosDbEx.Response{}} - The operation was successful. {:bad_request, %CosmosDbEx.Response{}} - The JSON body is invalid. {:storage_limit_reached, %CosmosDbEx.Response{}} - The operation could not be completed because the storage limit of the partition has been reached. {:conflict, %CosmosDbEx.Response{}} - The ID provided for the new document has been taken by an existing document. {:entity_too_large, %CosmosDbEx.Response{}} - The document size in the request exceeded the allowable document size. {:error, error} - Any errors encountered by our http client that aren't related to CosmosDb. Examples iex&gt; item = %{ name : &quot;ACME hair dryer&quot; , id : &quot;ACME-HD-WOLF01234&quot; , location : &quot;Bottom of a cliff&quot; } iex&gt; container = CosmosDbEx.Container . new ( &quot;database&quot; , &quot;container&quot; ) iex&gt; container |&gt; CosmosDbEx . create_document ( item , item . name ) { :ok , % CosmosDbEx.Response { body : %{ &quot;_attachments&quot; =&gt; &quot;attachments/&quot; , &quot;_etag&quot; =&gt; &quot;00000000-0000-0000-0000-000000000000&quot; , &quot;_rid&quot; =&gt; &quot;AAAAAAAAAAAAAAAAA==&quot; , &quot;_self&quot; =&gt; &quot;dbs/AAAAAA==/colls/AAAAAAAAAAA=/docs/AAAAAAAAAAAAAAAAAAAAAA==/&quot; , &quot;_ts&quot; =&gt; 1620141668 , &quot;id&quot; =&gt; &quot;ACME-HD-WOLF01234&quot; , &quot;location&quot; =&gt; &quot;Bottom of a cliff&quot; , &quot;name&quot; =&gt; &quot;ACME hair dryer&quot; } , properties : %{ request_charge : &quot;6.29&quot; , request_duration : &quot;5.328&quot; } } }","ref":"CosmosDbEx.html#create_document/3","title":"CosmosDbEx.create_document/3","type":"function"},{"doc":"Retrieve a document by it's document id and partition key. Examples iex&gt; container = CosmosDbEx.Container . new ( &quot;database&quot; , &quot;container&quot; ) iex&gt; item_id = &quot;00000000-0000-0000-0000-000000000000&quot; iex&gt; partition_key = item_id iex&gt; CosmosDbEx . get_document ( container , item_id , partition_key ) { :ok , % CosmosDbEx.Response { body : %{ &quot;_attachments&quot; =&gt; &quot;attachments/&quot; , &quot;_etag&quot; =&gt; &quot;&quot; 00000000 - 0000 - 0000 - 0000 - 000000000000 &quot;&quot; , &quot;_rid&quot; =&gt; &quot;AAAAAAAAAAAAAAAAA==&quot; , &quot;_self&quot; =&gt; &quot;dbs/AAAAAA==/colls/AAAAAAAAAAA=/docs/AAAAAAAAAAAAAAAAAAAAAA==/&quot; , &quot;_ts&quot; =&gt; 1620141668 , &quot;id&quot; =&gt; &quot;00000000-0000-0000-0000-000000000000&quot; , &quot;name&quot; =&gt; &quot;Test item&quot; } , properties : %{ request_charge : &quot;1&quot; , request_duration : &quot;0.585&quot; } } }","ref":"CosmosDbEx.html#get_document/3","title":"CosmosDbEx.get_document/3","type":"function"},{"doc":"Retrieve all documents in the container. This function defaults to retrieving 100 documents at a time (this is Cosmos Db's default). You can increase the amount of items returned by calling get_documents(container, max_item_count) with any number up to 1000 (Cosmos Dbs max). If Cosmos Db has more documents than the query returned there will be a Continuation Token present in the properties (has the key continuation_token ). You continue retrieving all the documents by using the Continuation Token in your next call to get_documents . See the second example Examples iex&gt; CosmosDbEx.Container . new ( &quot;database&quot; , &quot;container&quot; ) |&gt; CosmosDbEx . get_documents ( ) { :ok , % CosmosDbEx.Response { body : %{ &quot;Documents&quot; =&gt; [ %{ &quot;_attachments&quot; =&gt; &quot;attachments/&quot; , &quot;_etag&quot; =&gt; &quot;&quot; 0200 c45b - 0000 - 0200 - 0000 - 609166640000 &quot;&quot; , &quot;_rid&quot; =&gt; &quot;Hj8rAI2HN48BAAAAAAAAAA==&quot; , &quot;_self&quot; =&gt; &quot;dbs/Hj8rAA==/colls/Hj8rAI2HN48=/docs/Hj8rAI2HN48BAAAAAAAAAA==/&quot; , &quot;_ts&quot; =&gt; 1620141668 , &quot;id&quot; =&gt; &quot;d22d663e-6fa1-49af-98f8-df397f266999&quot; , &quot;name&quot; =&gt; &quot;Test item here&quot; } , %{ &quot;_attachments&quot; =&gt; &quot;attachments/&quot; , &quot;_etag&quot; =&gt; &quot;&quot; 02003980 - 0000 - 0200 - 0000 - 60916 f6d0000 &quot;&quot; , &quot;_rid&quot; =&gt; &quot;Hj8rAI2HN48CAAAAAAAAAA==&quot; , &quot;_self&quot; =&gt; &quot;dbs/Hj8rAA==/colls/Hj8rAI2HN48=/docs/Hj8rAI2HN48CAAAAAAAAAA==/&quot; , &quot;_ts&quot; =&gt; 1620143981 , &quot;id&quot; =&gt; &quot;bef3c1f3-3f66-49a3-ba77-6e8d0e641664&quot; , &quot;name&quot; =&gt; &quot;This is a test&quot; } ] } , count : 2 , properties : %{ continuation_token : nil , request_charge : &quot;1&quot; , request_duration : &quot;0.66&quot; } , resource_id : &quot;Hj8rAI2HN48=&quot; } } iex&gt; container = CosmosDbEx.Container . new ( &quot;database&quot; , &quot;container&quot; ) iex&gt; { :ok , response } = container |&gt; CosmosDbEx . get_documents ( 2 ) { :ok , % CosmosDbEx.Response { body : %{ &quot;Documents&quot; =&gt; [ %{ &quot;_attachments&quot; =&gt; &quot;attachments/&quot; , &quot;_etag&quot; =&gt; &quot;&quot; 0200 c45b - 0000 - 0200 - 0000 - 609166640000 &quot;&quot; , &quot;_rid&quot; =&gt; &quot;Hj8rAI2HN48BAAAAAAAAAA==&quot; , &quot;_self&quot; =&gt; &quot;dbs/Hj8rAA==/colls/Hj8rAI2HN48=/docs/Hj8rAI2HN48BAAAAAAAAAA==/&quot; , &quot;_ts&quot; =&gt; 1620141668 , &quot;id&quot; =&gt; &quot;d22d663e-6fa1-49af-98f8-df397f266999&quot; , &quot;name&quot; =&gt; &quot;Test item here&quot; } , %{ &quot;_attachments&quot; =&gt; &quot;attachments/&quot; , &quot;_etag&quot; =&gt; &quot;&quot; 02003980 - 0000 - 0200 - 0000 - 60916 f6d0000 &quot;&quot; , &quot;_rid&quot; =&gt; &quot;Hj8rAI2HN48CAAAAAAAAAA==&quot; , &quot;_self&quot; =&gt; &quot;dbs/Hj8rAA==/colls/Hj8rAI2HN48=/docs/Hj8rAI2HN48CAAAAAAAAAA==/&quot; , &quot;_ts&quot; =&gt; 1620143981 , &quot;id&quot; =&gt; &quot;bef3c1f3-3f66-49a3-ba77-6e8d0e641664&quot; , &quot;name&quot; =&gt; &quot;This is a test&quot; } ] } , count : 2 , properties : %{ continuation_token : %{ &quot;range&quot; =&gt; %{ &quot;max&quot; =&gt; &quot;FF&quot; , &quot;min&quot; =&gt; &quot;&quot; } , &quot;token&quot; =&gt; &quot;Hj8rAI2HN48CAAAAAAAAAA==&quot; } , request_charge : &quot;1&quot; , request_duration : &quot;0.429&quot; } , resource_id : &quot;Hj8rAI2HN48=&quot; } } iex&gt; { :ok , response } = container |&gt; CosmosDbEx . get_documents ( response . properties . continuation_token ) { :ok , % CosmosDbEx.Response { body : %{ &quot;Documents&quot; =&gt; [ %{ &quot;_attachments&quot; =&gt; &quot;attachments/&quot; , &quot;_etag&quot; =&gt; &quot;&quot; 82035043 - 0000 - 0200 - 0000 - 60 a1bac30000 &quot;&quot; , &quot;_rid&quot; =&gt; &quot;Hj8rAI2HN48DAAAAAAAAAA==&quot; , &quot;_self&quot; =&gt; &quot;dbs/Hj8rAA==/colls/Hj8rAI2HN48=/docs/Hj8rAI2HN48DAAAAAAAAAA==/&quot; , &quot;_ts&quot; =&gt; 1621211843 , &quot;id&quot; =&gt; &quot;2323490-23-23-3923493293&quot; , &quot;name&quot; =&gt; &quot;This is a test of the protocol&quot; } , %{ &quot;_attachments&quot; =&gt; &quot;attachments/&quot; , &quot;_etag&quot; =&gt; &quot;&quot; 8203015 f - 0000 - 0200 - 0000 - 60 a1c47e0000 &quot;&quot; , &quot;_rid&quot; =&gt; &quot;Hj8rAI2HN48FAAAAAAAAAA==&quot; , &quot;_self&quot; =&gt; &quot;dbs/Hj8rAA==/colls/Hj8rAI2HN48=/docs/Hj8rAI2HN48FAAAAAAAAAA==/&quot; , &quot;_ts&quot; =&gt; 1621214334 , &quot;id&quot; =&gt; &quot;ACME-HD-WOLF01234&quot; , &quot;location&quot; =&gt; &quot;Bottom of a cliff&quot; , &quot;name&quot; =&gt; &quot;ACME hair dryer&quot; } , %{ &quot;_attachments&quot; =&gt; &quot;attachments/&quot; , &quot;_etag&quot; =&gt; &quot;&quot; 5 d00e34c - 0000 - 0200 - 0000 - 60 b41c140000 &quot;&quot; , &quot;_rid&quot; =&gt; &quot;Hj8rAI2HN48GAAAAAAAAAA==&quot; , &quot;_self&quot; =&gt; &quot;dbs/Hj8rAA==/colls/Hj8rAI2HN48=/docs/Hj8rAI2HN48GAAAAAAAAAA==/&quot; , &quot;_ts&quot; =&gt; 1622416404 , &quot;id&quot; =&gt; &quot;TestDoc-2-1-3-2&quot; , &quot;location&quot; =&gt; &quot;Under da hill&quot; , &quot;name&quot; =&gt; &quot;Just another test document&quot; } ] } , count : 3 , properties : %{ continuation_token : nil , request_charge : &quot;1&quot; , request_duration : &quot;0.596&quot; } , resource_id : &quot;Hj8rAI2HN48=&quot; } }","ref":"CosmosDbEx.html#get_documents/2","title":"CosmosDbEx.get_documents/2","type":"function"},{"doc":"","ref":"CosmosDbEx.html#get_documents/3","title":"CosmosDbEx.get_documents/3","type":"function"},{"doc":"Sends a query to Cosmos Db. Parameters query: The query contains the SQL query text. params: A List of key/value pairs that correspond to values in the query. Query String and Params Notice the query string in the example below. For each value that will be substituted by an entry in the params list, the key in the query string must be annotated with an '@' symbol. Example: iex&gt; query_text = &quot;SELECT * FROM ItemsContainer c WHERE c.id = @id and c.name = @name&quot; iex&gt; params = [ { &quot;id&quot; , &quot;ACME-HD-WOLF01234&quot; } , { &quot;name&quot; , &quot;ACME hair dryer&quot; } ] iex&gt; CosmosDbEx . query ( query_text , params ) { :ok , % CosmosDbEx.Response { body : %{ &quot;Documents&quot; =&gt; [ %{ &quot;_attachments&quot; =&gt; &quot;attachments/&quot; , &quot;_etag&quot; =&gt; &quot;&quot; 8203015 f - 0000 - 0200 - 0000 - 60 a1c47e0000 &quot;&quot; , &quot;_rid&quot; =&gt; &quot;AAarArAAAAAFAAAAAAAAAA==&quot; , &quot;_ts&quot; =&gt; 1621214334 , &quot;id&quot; =&gt; &quot;ACME-HD-WOLF01234&quot; , &quot;location&quot; =&gt; &quot;Bottom of a cliff&quot; , &quot;name&quot; =&gt; &quot;ACME hair dryer&quot; } ] } , count : 1 , properties : %{ continuation_token : nil , request_charge : &quot;2.83&quot; , request_duration : &quot;0.734&quot; } , resource_id : &quot;AA8rAA2AN48=&quot; } }","ref":"CosmosDbEx.html#query/3","title":"CosmosDbEx.query/3","type":"function"},{"doc":"Provides all necessary functionality to peform Cosmos Db Rest API calls with authorization. The general authorization methods were created using the Azure documentation located here: Access control in the Azure Cosmos DB SQL API","ref":"CosmosDbEx.Auth.html","title":"CosmosDbEx.Auth","type":"module"},{"doc":"Generates a hashed token signature for a master token that will be used for authorization of each request to the Cosmos Db api. Params http_verb: A HTTP verb, such as GET, POST, PUT. resource_type: Identifies the type of resource that the request is for. Example: dbs, colls, docs. resource_id: The identity of the resource that the request is directed at. An example for a request for a collection would be: &quot;dbs/MyDatabase/colls/MyCollection date: The UTC date and time the message is being sent. The date must conform to the format defined in RFC 7231 Date/Time Formats . Note that this must also be the date passed in the x-ms-date header. Example: &quot;Tue, 01 Nov 1994 08:12:31 GMT&quot;. key: This is the Encoded key for your Cosmos Db database. It is usually either the primary or secondary key that can be found in the Keys setting in your databases blade. Note: This key should never be saved and controlled in any repo. The key should be retrieved from something like Azure Key Vault, or an environment variable. key_type: The type of key being used for authorization. This will normally be &quot;master&quot;. token_version: The version of the token, or the format rather, being sent to Cosmos Db. The current supported version is 1.0. Remarks I'm not really certain how much compute it takes to decode a Base64 encoded string, but there are actually Elixir libraries out there that drop down to 'C' just to make it faster. If it turns out that constantly decoding the key hampers performance then we can see about possibly storing the decoded key in ETS to reduce compute a little bit.","ref":"CosmosDbEx.Auth.html#generate_auth_signature/6","title":"CosmosDbEx.Auth.generate_auth_signature/6","type":"function"},{"doc":"A simple struct used to instruct the library which database and container to communicate with.","ref":"CosmosDbEx.Container.html","title":"CosmosDbEx.Container","type":"module"},{"doc":"","ref":"CosmosDbEx.Container.html#new/2","title":"CosmosDbEx.Container.new/2","type":"function"},{"doc":"","ref":"CosmosDbEx.Container.html#t:t/0","title":"CosmosDbEx.Container.t/0","type":"type"},{"doc":"","ref":"CosmosDbEx.Documents.Identifiable.html","title":"CosmosDbEx.Documents.Identifiable","type":"protocol"},{"doc":"Returns the id of the item.","ref":"CosmosDbEx.Documents.Identifiable.html#get_id/1","title":"CosmosDbEx.Documents.Identifiable.get_id/1","type":"function"},{"doc":"","ref":"CosmosDbEx.Documents.Identifiable.html#t:t/0","title":"CosmosDbEx.Documents.Identifiable.t/0","type":"type"},{"doc":"Formatted response from CosmosDb. Request Charge This is the R/U (Request Unit) charge that the query cost to return the response from Cosmos Db. In other words, this was the cost of all the database operations that had to happen in order for CosmosDb to read or write to/from the database. For more information on Request Units please see Request Units in Azure Cosmos DB . Request Duration This is the time, in milliseconds, that it took CosmosDb to execute the query sent. Body This is the body of the response sent from Cosmos Db. It is expected that the body will be a map. Additional items gathered from the response headers will be placed in the properties field of the Response struct. Some convience methods will be provided for commonly used properties like 'request charge', 'request duration', and 'continuation_token'.","ref":"CosmosDbEx.Response.html","title":"CosmosDbEx.Response","type":"module"},{"doc":"Returns the continuation token of the request. This token can be sent with the next request to retrieve the next page of results from the query. Example iex&gt; container = Container . new ( &quot;TestItemsDb&quot; , &quot;ItemsContainer&quot; ) iex&gt; { :ok , response } = CosmosDbEx . get_documents ( container ) iex&gt; { :ok , response } = CosmosDbEx . get_documents ( container , CosmosDbEx.Response . get_continuation_token ( response ) ) NOTE: Cosmos Db returns results in pages of up to a maximum of 1000 items. Returns nil if no continuation token is found. Nil also signals that there are no more items left from the query.","ref":"CosmosDbEx.Response.html#get_continuation_token/1","title":"CosmosDbEx.Response.get_continuation_token/1","type":"function"},{"doc":"Returns the request charge (Cosmos Db's Request Unit measurement) of the request. Returns nil if no request charge is found in the response.","ref":"CosmosDbEx.Response.html#get_request_charge/1","title":"CosmosDbEx.Response.get_request_charge/1","type":"function"},{"doc":"Returns the request duration, in milliseconds, of the request. Returns nil if no request_duration is found.","ref":"CosmosDbEx.Response.html#get_request_duration/1","title":"CosmosDbEx.Response.get_request_duration/1","type":"function"},{"doc":"","ref":"CosmosDbEx.Response.html#t:t/0","title":"CosmosDbEx.Response.t/0","type":"type"}]